@@ -480,6 +480,13 @@ void U_EMRRECTANGLE_draw(const char *contents, FILE *out,
     POINT_D dim;
     dim.x = RB.x - LT.x;
     dim.y = RB.y - LT.y;
+
+    // If fixBrokenYTransform is true, we have to flip points on the Y axis.
+    if (states->fixBrokenYTransform && dim.y < 0) {
+        dim.y = -dim.y;
+        LT.y -= dim.y;
+    }
+
     fprintf(out,
             "<%srect x=\"%.4f\" y=\"%.4f\" width=\"%.4f\" height=\"%.4f\" ",
             states->nameSpaceString, LT.x, LT.y, dim.x, dim.y);
@@ -506,15 +513,19 @@ void U_EMRROUNDRECT_draw(const char *contents, FILE *out,
     POINT_D RB = point_cal(states, (double)pEmr->rclBox.right,
                            (double)pEmr->rclBox.bottom);
     POINT_D dim;
-    POINT_D round;
     dim.x = RB.x - LT.x;
     dim.y = RB.y - LT.y;
+
+    // If fixBrokenYTransform is true, we have to flip points on the Y axis.
+    if (states->fixBrokenYTransform && dim.y < 0) {
+        dim.y = -dim.y;
+        LT.y -= dim.y;
+    }
+
     fprintf(out,
             "<%srect x=\"%.4f\" y=\"%.4f\" width=\"%.4f\" height=\"%.4f\" ",
             states->nameSpaceString, LT.x, LT.y, dim.x, dim.y);
-    round = point_cal(states, (double)pEmr->szlCorner.cx,
-                      (double)pEmr->szlCorner.cy);
-    fprintf(out, "rx=\"%.4f\" ry=\"%.4f\" ", round.x, round.y);
+    fprintf(out, "rx=\"%.4f\" ry=\"%.4f\" ", scaleX(states, (double)pEmr->szlCorner.cx), scaleX(states, (double)pEmr->szlCorner.cy));
     bool filled = false;
     bool stroked = false;
     fill_draw(states, out, &filled, &stroked);
